"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScreencastFrameCollector = void 0;
const tslib_1 = require("tslib");
const debug_1 = require("debug");
const events_1 = require("events");
const utils_1 = require("./utils");
const debug = debug_1.default('pw-video:ScreencastFrameCollector');
class ScreencastFrameCollector extends events_1.EventEmitter {
    constructor(page, options) {
        super();
        this._originalPage = page;
        this._clients = [];
        this._followPopups = options ? options.followPopups : false;
        this._popupFollower = this._popupFollower.bind(this);
    }
    static create(originalPage, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            utils_1.ensurePageType(originalPage);
            const collector = new ScreencastFrameCollector(originalPage, options);
            return collector;
        });
    }
    _popupFollower(popup) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this._activatePage(popup);
            this.emit('popupFollowed');
            popup.once('close', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield this._deactivatePage(popup);
            }));
        });
    }
    _installPopupFollower(page) {
        page.on('popup', this._popupFollower);
    }
    _uninstallPopupFollower(page) {
        page.off('popup', this._popupFollower);
    }
    _buildClient(page) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const context = page.context();
            const client = yield context.newCDPSession(page);
            return client;
        });
    }
    _getActiveClient() {
        return this._clients[this._clients.length - 1];
    }
    _listenForFrames(client) {
        this._endedPromise = new Promise((resolve) => {
            client.on('Page.screencastFrame', (payload) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                if (!payload.metadata.timestamp) {
                    debug('skipping frame without timestamp');
                    return;
                }
                if (this._stoppedTimestamp && payload.metadata.timestamp > this._stoppedTimestamp) {
                    debug('all frames received');
                    resolve();
                    return;
                }
                debug(`received frame with timestamp ${payload.metadata.timestamp}`);
                const ackPromise = client.send('Page.screencastFrameAck', {
                    sessionId: payload.sessionId,
                });
                this.emit('screencastframe', {
                    data: Buffer.from(payload.data, 'base64'),
                    received: Date.now(),
                    timestamp: payload.metadata.timestamp,
                });
                try {
                    yield ackPromise;
                }
                catch (e) {
                    debug('error sending screencastFrameAck %j', e.message);
                }
            }));
        });
    }
    _activatePage(page) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            debug('activating page: ', page.url());
            const previousClient = this._getActiveClient();
            if (previousClient) {
                yield previousClient.send('Page.stopScreencast');
            }
            const client = yield this._buildClient(page);
            this._clients.push(client);
            this._listenForFrames(client);
            yield client.send('Page.startScreencast', {
                everyNthFrame: 1,
            });
        });
    }
    _deactivatePage(page) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            debug('deactivating page: ', page.url());
            this._clients.pop();
            const previousClient = this._getActiveClient();
            try {
                yield previousClient.send('Page.startScreencast', {
                    everyNthFrame: 1,
                });
            }
            catch (e) {
                debug('error reactivating original page %j', e.message);
            }
        });
    }
    start() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            debug('start');
            yield this._activatePage(this._originalPage);
            if (this._followPopups) {
                this._installPopupFollower(this._originalPage);
            }
        });
    }
    stop() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this._stoppedTimestamp) {
                throw new Error('pw-video: Cannot call stop twice on the same capture.');
            }
            if (this._followPopups) {
                this._uninstallPopupFollower(this._originalPage);
            }
            this._stoppedTimestamp = Date.now() / 1000;
            debug(`stopping screencast at ${this._stoppedTimestamp}`);
            yield Promise.race([
                this._endedPromise,
                new Promise((resolve) => setTimeout(resolve, 1000)),
            ]);
            try {
                debug('detaching client');
                for (const client of this._clients) {
                    yield client.detach();
                }
            }
            catch (e) {
                debug('error detaching client', e.message);
            }
            debug('stopped');
            return this._stoppedTimestamp;
        });
    }
}
exports.ScreencastFrameCollector = ScreencastFrameCollector;
//# sourceMappingURL=ScreencastFrameCollector.js.map